@@ -3,6 +3,7 @@
 import  os
 import  sys
 import  platform
+import  getpass
 
 from    subprocess import check_call, DEVNULL, STDOUT, Popen, PIPE
 from    datetime import datetime
@@ -12,33 +13,88 @@ class BootManager(object):
        Currently supports the following platforms
        Linux"""
 
-    LINUX_OS_NAME = "Linux"
+    LINUX_OS_NAME       = "Linux"
+    ENABLE_CMD_OPT      = "--enable_auto_start"
+    DISABLE_CMD_OPT     = "--disable_auto_start"
+    CHECK_CMD_OPT       = "--check_auto_start"
 
-    def __init__(self, uio=None, allowRootUser=False):
+    @staticmethod
+    def AddCmdArgs(parser):
+        """@brief Add cmd line arguments to enable, disable and show the systemd boot state.
+           @param parser An instance of argparse.ArgumentParser."""
+        parser.add_argument(BootManager.ENABLE_CMD_OPT,  help="Auto start when this computer starts.", action="store_true", default=False)
+        parser.add_argument(BootManager.DISABLE_CMD_OPT, help="Disable auto starting when this computer starts.", action="store_true", default=False)
+        parser.add_argument(BootManager.CHECK_CMD_OPT,   help="Check the status of an auto started icons_gw instance.", action="store_true", default=False)
+
+    @staticmethod
+    def HandleOptions(uio, options, enable_syslog):
+        """@brief Handle one of the bot manager command line options if the 
+                  user passed it on the cmd line.
+           @param uio A UIO instance.
+           @param options As returned from parser.parse_args() where parser 
+                          is an instance of argparse.ArgumentParser.
+           @param enable_syslog True to enable systemd syslog output.
+           @return True if handled , False if not."""
+        handled = False
+        if options.check_auto_start:
+            BootManager.CheckAutoStartStatus(uio)
+            handled = True
+            
+        elif options.enable_auto_start:
+            BootManager.EnableAutoStart(uio, enable_syslog)
+            handled = True
+            
+        elif options.disable_auto_start:
+            BootManager.DisableAutoStart(uio)
+            handled = True
+
+        return handled
+
+    @staticmethod
+    def EnableAutoStart(uio, enable_syslog):
+        """@brief Enable this program to auto start when the computer on which it is installed starts."""
+        bootManager = BootManager(uio=uio, ensureRootUser=True)
+        arsString = " ".join(sys.argv)
+        bootManager.add(argString=arsString, enableSyslog=enable_syslog)
+
+    @staticmethod
+    def DisableAutoStart(uio):
+        """@brief Enable this program to auto start when the computer on which it is installed starts."""
+        bootManager = BootManager(uio=uio, ensureRootUser=True)
+        bootManager.remove()
+        
+    @staticmethod
+    def CheckAutoStartStatus(uio):
+        """@brief Check the status of a process previously set to auto start."""
+        bootManager = BootManager(uio=uio)
+        lines = bootManager.getStatus()
+        if lines and len(lines) > 0:
+            for line in lines:
+                uio.info(line)
+        
+    def __init__(self, uio=None, allowRootUser=True, ensureRootUser=False):
         """@brief Constructor
            @param uio A UIO instance to display user output. If unset then no output
                   is displayed to user.
            @param allowRootUser If True then allow root user to to auto start
-                  programs. We do not do this by default because often a programs
-                  configuration is stored in the users home path. E.G a config
-                  file in ~/.<configfilename> or ~/.config/<configfolder or filename>.
-                  If a program is started as root user then this config may not
-                  be present in the /root folder and so when the program is started
-                  it's config will not be found. If a program stores no config in
-                  the home folder then this is not an issue."""
+                  programs. Note that as the BootManager is responsible for 
+                  ensuring programs are started up when a machine boots up 
+                  the installed program should be installed for the root 
+                  user on Linux systems.
+           @param ensureRootUser If True the current user must be root user (Linux systems)."""
         self._uio = uio
         self._allowRootUser=allowRootUser
         self._osName = platform.system()
         self._platformBootManager = None
         if self._osName == BootManager.LINUX_OS_NAME:
-            self._platformBootManager = LinuxBootManager(uio, self._allowRootUser)
+            self._platformBootManager = LinuxBootManager(uio, self._allowRootUser, ensureRootUser)
         else:
             raise Exception("{} is an unsupported OS.".format(self._osName) )
 
-    def add(self, user, argString=None, enableSyslog=False):
+    def add(self, user=None, argString=None, enableSyslog=False):
         """@brief Add an executable file to the processes started at boot time.
            @param exeFile The file/program to be executed. This should be an absolute path.
-           @param user The user that will run the executable file.
+           @param user The user that will run the executable file. If left as None then the current user will be used.
            @param argString The argument string that the program is to be launched with.
            @param enableSyslog If True enable stdout and stderr to be sent to syslog."""
         if self._platformBootManager:
@@ -63,23 +119,67 @@ class BootManager(object):
 class LinuxBootManager(object):
     """@brief Responsible for adding/removing Linux services using systemd."""
 
-    LOG_PATH="/var/log"
-    SERVICE_FOLDER = "/etc/systemd/system/"
-    SYSTEM_CTL = "/bin/systemctl"
+    LOG_PATH            ="/var/log"
+    ROOT_SERVICE_FOLDER = "/etc/systemd/system/"
+    SYSTEM_CTL_1        = "/bin/systemctl"
+    SYSTEM_CTL_2        = "/usr/bin/systemctl"
 
-    def __init__(self, uio, allowRootUser):
+    @staticmethod
+    def GetSystemCTLBin():
+        """@brief Get the location of the systemctl binary file on this system.
+           @return The systemctl bin file."""
+        binFile = None
+        if os.path.isfile(LinuxBootManager.SYSTEM_CTL_1):
+            binFile = LinuxBootManager.SYSTEM_CTL_1
+        elif os.path.isfile(LinuxBootManager.SYSTEM_CTL_2):
+            binFile = LinuxBootManager.SYSTEM_CTL_2
+        else:
+            raise Exception("Failed to find the location of the systemctl bin file on this machine.")
+        return binFile
+
+    @staticmethod
+    def GetServiceFolder(rootUser):
+        """"@brief Get the service folder to use.
+            @param rootUser False if non root user.
+            @return The folder that should hold the systemctl service files."""
+        serviceFolder = None
+        if rootUser:
+            serviceFolder = LinuxBootManager.ROOT_SERVICE_FOLDER
+        else:
+            homeFolder = os.path.expanduser('~')
+            serviceFolder = os.path.join(homeFolder, '.config/systemd/user/')
+            if not os.path.isdir(serviceFolder):
+                os.makedirs(serviceFolder)
+
+        if not os.path.isdir(serviceFolder):
+            raise Exception(f"{serviceFolder} folder not found.")
+        return serviceFolder
+
+    def __init__(self, uio, allowRootUser, ensureRootUser):
         """@brief Constructor
            @param uio A UIO instance to display user output. If unset then no output is displayed to user.
-           @param allowRootUser If True then allow root user to to auto start programs."""
+           @param allowRootUser If True then allow root user to to auto start programs.
+           @param ensureRootUser If True the current user must be root user."""
         self._uio = uio
         self._logFile = None
         self._allowRootUser=allowRootUser
-
-        if os.geteuid() != 0:
-            self._fatalError("Please run this command with root level access.")
-
         self._info("OS: {}".format(platform.system()) )
+        self._rootMode = False # If True run as root, else False.
+        self._systemCtlBin = LinuxBootManager.GetSystemCTLBin()
+        if ensureRootUser and os.geteuid() != 0:
+            self._fatalError(self.__class__.__name__ + ": Not root user. Ensure that you are root user and try again.")
 
+        if os.geteuid() == 0:
+            if not allowRootUser:
+                self._fatalError(self.__class__.__name__ + f": You are running as root user but allowRootUser={allowRootUser}.")
+            else:
+                self._rootMode = True
+        if not self._rootMode:
+            self._cmdLinePrefix = self._systemCtlBin + " --user"
+        else:
+            self._cmdLinePrefix = self._systemCtlBin
+        self._username = getpass.getuser()
+        self._serviceFolder = LinuxBootManager.GetServiceFolder(self._rootMode)
         self._appName = None
 
     def _getInstallledStartupScript(self):
@@ -87,41 +187,14 @@ class LinuxBootManager(object):
                   named the same as the python file executed without the .py suffix.
            @return The startup script file (absolute path)."""""
         startupScript=None
-        pythonFile = sys.argv[0] # The python file executed at program startup
-        if pythonFile.startswith("./"):
-            pythonFile=pythonFile[2:]
-            
-        envPaths = self._getPaths()
-
-        # Search first for a file that does not have the .py suffix as 
-        # this may be installed via a deb/rpm installation.
-        exeFile=os.path.basename( pythonFile.replace(".py", "") )
-        if envPaths and len(envPaths) > 0:
-            for envPath in envPaths:
-                _exeStartupScript = os.path.join(envPath, exeFile)
-                if os.path.isfile(_exeStartupScript):
-                    startupScript=_exeStartupScript
-                    break 
-
-        # If the script file has not been found then search for a file with 
-        # the .py suffix.       
-        if not startupScript:
-            if envPaths and len(envPaths) > 0:
-                for envPath in envPaths:
-                    _startupScript = os.path.join(envPath, pythonFile)
-                    if os.path.isfile(_startupScript):
-                        startupScript=_startupScript
-                        break     
-                      
-        if not startupScript:
-            paths = self._getPaths()
-            if len(paths):
-                for _path in paths:
-                    self._info(_path)
-                self._fatalError("{} startup script not found using the PATH env var".format(pythonFile) )
-            else:
-                self._fatalError("No PATH env var found.")
-
+        argList = sys.argv
+        # If the first argument in the arg is a file.
+        if len(argList) > 0:
+            firstArg = argList[0]
+            if os.path.isfile(firstArg) or os.path.islink(firstArg):
+                startupScript = firstArg
+        if startupScript is None:
+            raise Exception("Failed to find the startup script.")
         return startupScript
 
     def _getPaths(self):
@@ -188,7 +261,9 @@ class LinuxBootManager(object):
             self._fatalError("{} file not found.".format(absApp) )
 
         appName = appName.replace(".py", "")
-        self._logFile = os.path.join(LinuxBootManager.LOG_PATH, appName)
+        if self._rootMode:
+            # We can only save to /var/log/ is we are root user.
+            self._logFile = os.path.join(LinuxBootManager.LOG_PATH, appName)
 
         return (appName, absApp)
 
@@ -197,7 +272,7 @@ class LinuxBootManager(object):
            @param appName The name of the app to execute.
            @return The absolute path to the service file """
         serviceName = "{}.service".format(appName)
-        serviceFile = os.path.join(LinuxBootManager.SERVICE_FOLDER, serviceName)
+        serviceFile = os.path.join(self._serviceFolder, serviceName)
         return serviceFile
 
     def add(self, user, argString=None, enableSyslog=False):
@@ -209,8 +284,11 @@ class LinuxBootManager(object):
                        to non root user paths on Linux systems and the startup
                        script should then be executed with the same username in
                        order that the same config file is used.
+                       If set to None then the current user is used.
            @param argString The argument string that the program is to be launched with.
            @param enableSyslog If True enable stdout and stderr to be sent to syslog."""
+        if user is None:
+            user = self._username
 
         appName, absApp = self._getApp()
 
@@ -231,13 +309,21 @@ class LinuxBootManager(object):
         else:
             lines.append("StandardOutput=null")
             lines.append("StandardError=journal")
-        lines.append("User={}".format(user))
+        if self._rootMode and user != 'root':
+            lines.append("User={}".format(user))
 
         #We add the home path env var so that config files (if stored in/under 
         # the users home dir) can be found by the prgram.
         if user and len(user) > 0:
             lines.append('Environment="HOME=/home/{}"'.format(user))
         if argString:
+            argString = argString.strip()
+            if argString.startswith(absApp):
+                argString=argString.replace(absApp, "")
+            # We don't want the enable cmd opt in the cmd we add to the systemd file.
+            if argString.find(BootManager.ENABLE_CMD_OPT):
+                argString = argString.replace(BootManager.ENABLE_CMD_OPT, "")
+            argString = argString.strip()
             lines.append("ExecStart={} {}".format(absApp, argString))
         else:
             lines.append("ExecStart={}".format(absApp))
@@ -250,15 +336,16 @@ class LinuxBootManager(object):
             fd = open(serviceFile, 'w')
             fd.write( "\n".join(lines) )
             fd.close()
+            self._info(f"Created {serviceFile}")
         except IOError:
             self._fatalError("Failed to create {}".format(serviceFile) )
 
-        cmd = "{} daemon-reload".format(LinuxBootManager.SYSTEM_CTL)
+        cmd = "{} daemon-reload".format(self._cmdLinePrefix)
         self._runLocalCmd(cmd)
-        cmd = "{} enable {}".format(LinuxBootManager.SYSTEM_CTL, appName)
+        cmd = "{} enable {}".format(self._cmdLinePrefix, appName)
         self._info("Enabled {} on restart".format(appName))
         self._runLocalCmd(cmd)
-        cmd = "{} start {}".format(LinuxBootManager.SYSTEM_CTL, appName)
+        cmd = "{} start {}".format(self._cmdLinePrefix, appName)
         self._runLocalCmd(cmd)
         self._info("Started {}".format(appName))
 
@@ -270,11 +357,11 @@ class LinuxBootManager(object):
 
         serviceFile = self._getServiceFile(appName)
         if os.path.isfile(serviceFile):
-            cmd = "{} disable {}".format(LinuxBootManager.SYSTEM_CTL, appName)
+            cmd = "{} disable {}".format(self._cmdLinePrefix, appName)
             self._runLocalCmd(cmd)
             self._info("Disabled {} on restart".format(appName))
 
-            cmd = "{} stop {}".format(LinuxBootManager.SYSTEM_CTL, appName)
+            cmd = "{} stop {}".format(self._cmdLinePrefix, appName)
             self._runLocalCmd(cmd)
             self._info("Stopped {}".format(appName))
 
@@ -286,14 +373,12 @@ class LinuxBootManager(object):
     def getStatusLines(self):
         """@brief Get a status report.
            @return Lines of text indicating the status of a previously started process."""
-        statusLines = []
         appName, _ = self._getApp()
-        cmd = "{} status {}".format(LinuxBootManager.SYSTEM_CTL, appName)
-        p = Popen([LinuxBootManager.SYSTEM_CTL, 'status', appName], stdin=PIPE, stdout=PIPE, stderr=PIPE)
+        if self._rootMode:
+            p = Popen([self._systemCtlBin, 'status', appName], stdin=PIPE, stdout=PIPE, stderr=PIPE)
+        else:
+            p = Popen([self._systemCtlBin, '--user', 'status', appName], stdin=PIPE, stdout=PIPE, stderr=PIPE)
         output, err = p.communicate(b"input data that is passed to subprocess' stdin")
         response = output.decode() + "\n" + err.decode()
         lines = response.split("\n")
         return lines
-            
-            
-            
\ No newline at end of file
