@@ -1,4 +1,4 @@
-#!/bin/sh/env python3d
+#!/bin/sh/env python3
 
 import  os
 import  base64
@@ -6,6 +6,7 @@ import  datetime
 import  sys
 import  json
 import  copy
+import  platform
 
 from    shutil import copyfile
 
@@ -313,39 +314,36 @@ class ConfigManager(object):
         if not cfgFilename:
             raise Exception("No config filename defined.")
 
-        cfgFilename = cfgFilename
         if addDotToFilename:
             if not cfgFilename.startswith("."):
 
                 cfgFilename=".%s" % (cfgFilename)
 
-            else:
-
-                cfgFilename=cfgFilename
-
         #The the config path has been set then use it
         if cfgPath:
             configPath = cfgPath
 
         else:
-            configPath=""
-            #If an absolute path is set for the config file then don't try to
-            #put the file in the users home dir
-            if not cfgFilename.startswith("/"):
-                configPath = expanduser("~")
-                configPath = configPath.strip()
-                #If no user is known then default to root user.
-                #This occurs on Omega2 startup apps.
-                if len(configPath) == 0 or configPath == '/' or configPath == '/root':
-                    configPath="/root"
+            # If the root user on a Linux system
+            if platform.system() == 'Linux' and os.geteuid() == 0:
+                # This should be the root users config folder.
+                configPath="/root"
+
+            else:
+                configPath=""
+                #If an absolute path is set for the config file then don't try to
+                #put the file in the users home dir
+                if not cfgFilename.startswith("/"):
+                    configPath = expanduser("~")
+                    configPath = configPath.strip()
 
         return join( configPath, cfgFilename )
-
+    
     def __init__(self, uio, cfgFilename, defaultConfig, addDotToFilename=True, encrypt=False, cfgPath=None):
         """@brief Constructor
            @param uio A UIO (User Input Output) instance. May be set to None if no user messages are required.
-           @param cfgFilename   The name of the config file.
-           @param defaultConfig A default config instance containg all the default key-value pairs.
+           @param cfgFilename   The name of the config file. If this is None then the default config filename is used.
+           @param defaultConfig A default config instance containing all the default key-value pairs.
            @param addDotToFilename If True (default) then a . is added to the start of the filename. This hides the file in normal cases.
            @param encrypt If True then data will be encrypted in the saved files.
                           The encryption uses part of the the local SSH RSA private key.
@@ -362,6 +360,10 @@ class ConfigManager(object):
         self._cfgPath           = cfgPath
         self._configDict        = {}
 
+        # If the user passed None in as the cfg filename then generate the default config file.
+        if self._cfgFilename is None:
+            self._cfgFilename =  ConfigManager.GetDefaultConfigFilename()
+
         self._cfgFile = self._getConfigFile()
         self._modifiedTime = self._getModifiedTime()
 
@@ -472,18 +474,28 @@ class ConfigManager(object):
             #Get the config from the stored config file.
             loadedConfig = self._getDict()
             #Get list of all the keys from the config file loaded.
-            loadedConfigKeys = loadedConfig.keys()
-            #Get the default config
-            self._configDict = copy.deepcopy( self._defaultConfig )
-            #Ensure that the config we use has all the values from the file we just loaded and any other values
-            #not in the config file loaded but defined in the default config are set to default values.
-            for key in loadedConfigKeys:
-                if key in self._configDict:
-                    self._configDict[key] = loadedConfig[key]
-                    self._debug("{} = {}".format(key, self._configDict[key]))
-                else:
-                    self._debug("----------> DROPPED FROM CONFIG: {} = {}".format(key, loadedConfig[key]))
+            loadedConfigKeys = list(loadedConfig.keys())
 
+            #Get the default config
+            defaultConfig = copy.deepcopy( self._defaultConfig )
+            defaultConfigKeys = list(defaultConfig.keys())
+
+            # Config parameters may be added or dropped over time. We use the default config to
+            # check for parameters that should be added/removed.
+
+            # Add any missing keys to the loaded config from the default config.
+            for defaultKey in defaultConfigKeys:
+                if defaultKey not in loadedConfigKeys:
+                    loadedConfig[defaultKey] = self._defaultConfig[defaultKey]
+                    self._debug("----------> DEFAULT VALUE ADDED: {} = {}".format(defaultKey, loadedConfig[defaultKey]))
+
+            # If some keys have been dropped from the config, remove them.
+            for loadedConfigKey in loadedConfigKeys:
+                if loadedConfigKey not in defaultConfigKeys:
+                    self._debug("----------> DROPPED FROM CONFIG: {} = {}".format(loadedConfigKey, loadedConfig[loadedConfigKey]))
+                    loadedConfig.pop(loadedConfigKey, None)
+
+            self._configDict = loadedConfig
         self._info("Loaded config from %s" % (self._cfgFile) )
 
     def inputFloat(self, key, prompt, minValue=UNSET_VALUE, maxValue=UNSET_VALUE):
@@ -737,3 +749,90 @@ class ConfigAttrDetails(object):
         self.minValue       =   minValue    #Only used for numbers
         self.maxValue       =   maxValue    #Only used for numbers
         self.allowEmpty     =   allowEmpty  #Only used for strings
+
+class DotConfigManager(ConfigManager):
+    """@brief This extends the previous ConfigManager and stores config under the ~/.config folder
+              rather than in the home folder (~) using a filename prefixed with the . character.
+              The ~/.config folder holds either a single config file of the startup python filename.
+              The ./config folder can contain another python module folder which contains the config
+              file. E.G for this example app the ~/.config/examples/pconfig_example.cfg folder is used."""
+    DEFAULT_CONFIG = None       # This must be overridden in a subclass to define the configuration parameters and values.
+    KEY_EDIT_ORDER_LIST = None
+
+    @staticmethod
+    def GetDefaultConfigFilename():
+        """@brief Get the default name of the config file for this app. This will be the program name
+                  (file that started up initially) without the .py extension. A .cfg extension is added
+                  and it will be found in the ~/.config folder."""
+        dotConfigFolder = '.config'
+        if platform.system() == 'Linux' and os.geteuid() == 0:
+            homePath = "/root"
+        else:
+            homePath = os.path.expanduser("~")
+        configFolder = os.path.join(homePath, dotConfigFolder)
+
+        if not os.path.isdir(homePath):
+            raise Exception(f"{homePath} HOME path does not exist.")
+        
+        # Create the ~/.config folder if it does not exist
+        if not os.path.isdir(configFolder):
+            # Create the ~/.config folder
+            os.makedir(configFolder)
+
+        progName = sys.argv[0]
+        if progName.endswith('.py'):
+            progName = progName[0:-3]
+        progName = os.path.basename(progName).strip()
+
+        # Note that we assume that addDotToFilename in the ConfigManager constructor is set True
+        # as this will prefix the filename with the . character.
+        if os.path.isdir(configFolder):
+            configFilename = os.path.join(".config", progName + ".cfg")
+
+        else:
+            raise Exception(f"Failed to create the {configFolder} folder.")
+
+        return configFilename
+
+    def __init__(self, defaultConfig, keyEditOrderList=None, uio=None, encrypt=False):
+        """@brief Constructor
+           @param defaultConfig A default config instance containing all the default key-value pairs.
+           @param keyEditOrderList A list of all the dict keys in the order that the caller wishes them to be displayed top the user.
+           @param uio A UIO (User Input Output) instance. May be set to None if no user messages are required.
+           @param encrypt If True then data will be encrypted in the saved files.
+                          The encryption uses part of the the local SSH RSA private key.
+                          This is not secure but assuming the private key has not been compromised it's
+                          probably the best we can do. 
+                          !!! Therefore if encrypt is set True then the an ssh key must be present !!!
+                          ||| in the ~/.ssh folder named id_rsa.                                   !!!"""
+        super().__init__(uio, DotConfigManager.GetDefaultConfigFilename(), defaultConfig, encrypt=encrypt)
+        self._keyEditOrderList = keyEditOrderList
+        # Ensure the config file is present and loaded into the internal dict.
+        self.load()
+
+    def show(self):
+        """@brief A helper method to show the dictionary config to the user.
+           @return A dictionary mapping the attribute ID's (keys) to dictionary keys (values)."""
+        maxKeyLen=10
+        # If the caller wants to present the parameters to the user in a partiular order
+        if self._keyEditOrderList:
+            keys = self._keyEditOrderList
+            defaultKeys = list(self._configDict.keys())
+            for defaultKey in defaultKeys:
+                if defaultKey not in keys:
+                    raise Exception(f"BUG: DotConfigManager: {defaultKey} key is missing from the keyEditOrderList.")
+        # Present the parameters to the user in any order.
+        else:
+            keys = list(self._configDict.keys())
+
+        for key in keys:
+            if len(key) > maxKeyLen:
+                maxKeyLen = len(key)
+        self._info("ID  PARAMETER"+" "*(maxKeyLen-8)+" VALUE")
+        id=1
+        idKeyDict = {}
+        for key in keys:
+            idKeyDict[id]=key
+            self._info("{:<3d} {:<{}} {}".format(id, key, maxKeyLen+1, self._configDict[key]))
+            id=id+1
+        return idKeyDict
