@@ -122,6 +122,7 @@ class SSH(object):
     PRIVATE_KEY_FILE_LIST               = ["id_rsa", "id_dsa", 'id_ecdsa']
     PUBLIC_KEY_FILE_LIST                = ["id_rsa.pub", "id_dsa.pub", 'id_ecdsa.pub']
     LOCAL_SSH_CONFIG_PATH               = os.path.join(os.path.expanduser("~"), ".ssh")
+    DEFAULT_REMOTE_SSH_CONFIG_FOLDER    = "~/.ssh"
     DEFAULT_REMOTE_SSH_AUTH_KEYS_FILE   = "~/.ssh/authorized_keys"
     DROPBEAR_DIR                        = "/etc/dropbear"
     DROPBEAR_AUTH_KEYS_FILE             = "%s/authorized_keys" % (DROPBEAR_DIR)
@@ -331,8 +332,10 @@ class SSH(object):
                         'timeout': timeout,
                         'username': self._username,
                         'key_filename': privateKeyFile,
-                        # This is required of else loging in without the password fails.
-                        'disabled_algorithms': dict(pubkeys=['rsa-sha2-256', 'rsa-sha2-512'])
+                        # This used to be required or else loging in without the password would fail.
+                        # This is no longer true for the latest paramiko as of 8 Mar 2024.
+                        # Therefore this workaround has been removed but shown in place in case of future issues.
+                        #'disabled_algorithms': dict(pubkeys=['rsa-sha2-256', 'rsa-sha2-512'])
                     }
                     # If we have a password then add this to the config
                     if self._password and len(self._password) > 0:
@@ -340,6 +343,12 @@ class SSH(object):
                     self._ssh.connect(**cfg)
                     connected = True
                     break
+        
+                # Ensure we throw an exception in the event of authencication failure as this ensures
+                # that the code to triggers the code to allow the user to copy thier public ssh key to
+                # the server in order that future logins are passwordless.
+                except AuthenticationException:
+                    raise
 
                 except:
                     pass
@@ -512,8 +521,17 @@ class SSH(object):
 
         cmd = "test -d %s" % (SSH.DROPBEAR_DIR)
         rc, stdoutLines, stderrLines = self.runCmd(cmd, throwError=False)
+        # If the ssh server uses dropbear
         if rc == 0:
             authKeysFile = SSH.DROPBEAR_AUTH_KEYS_FILE
+        else:
+            # Check for the users ssh config folder
+            cmd = "test -d %s" % (SSH.DEFAULT_REMOTE_SSH_CONFIG_FOLDER)
+            rc, stdoutLines, stderrLines = self.runCmd(cmd, throwError=False)
+            if rc != 0:
+                # If the remote ssh config folder does not exist, create it
+                cmd = f"mkdir {SSH.DEFAULT_REMOTE_SSH_CONFIG_FOLDER}"
+                rc, stdoutLines, stderrLines = self.runCmd(cmd, throwError=True)
 
         return authKeysFile
 
@@ -682,7 +700,7 @@ class SSHTunnelManager(object):
         forwardingServer = ForwardingServer(('', serverPort), SubHander)
         self._forwardingServerList.append(forwardingServer)
         newThread = threading.Thread(target=forwardingServer.serve_forever)
-        newThread.setDaemon(True)
+        newThread.daemon = True
         newThread.start()
 
     def stopFwdSSHTunnel(self, serverPort):
